@@ -1,22 +1,23 @@
-/*  Tarlz - Archiver with multimember lzip compression
-    Copyright (C) 2013-2019 Antonio Diaz Diaz.
+/* Tarlz - Archiver with multimember lzip compression
+   Copyright (C) 2013-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #define _FILE_OFFSET_BITS 64
 
+#include <cerrno>
 #include <climits>
 #include <cstdio>
 #include <cstdlib>
@@ -24,7 +25,7 @@
 #include <ctime>
 #include <string>
 #include <vector>
-#include <pthread.h>
+#include <pthread.h>	// for tarlz.h
 #include <stdint.h>
 #include <sys/stat.h>
 
@@ -180,21 +181,60 @@ bool show_member_name( const Extended & extended, const Tar_header header,
   }
 
 
-bool check_skip_filename( const Arg_parser & parser,
+bool check_skip_filename( const Cl_options & cl_opts,
                           std::vector< char > & name_pending,
-                          const char * const filename, const int filenames )
+                          const char * const filename )
   {
   if( Exclude::excluded( filename ) ) return true;	// skip excluded files
-  bool skip = filenames > 0;
+  bool skip = cl_opts.filenames > 0;
   if( skip )
-    for( int i = 0; i < parser.arguments(); ++i )
-      if( !parser.code( i ) && parser.argument( i ).size() )
+    for( int i = 0; i < cl_opts.parser.arguments(); ++i )
+      if( nonempty_arg( cl_opts.parser, i ) )
         {
         const char * const name =
-          remove_leading_dotslash( parser.argument( i ).c_str() );
+          remove_leading_dotslash( cl_opts.parser.argument( i ).c_str() );
         if( compare_prefix_dir( name, filename ) ||
             compare_tslash( name, filename ) )
           { skip = false; name_pending[i] = false; break; }
         }
   return skip;
   }
+
+
+mode_t get_umask()
+  {
+  static mode_t mask = 0;		// read once, cache the result
+  static bool first_call = true;
+  if( first_call ) { first_call = false; mask = umask( 0 ); umask( mask );
+                     mask &= S_IRWXU | S_IRWXG | S_IRWXO; }
+  return mask;
+  }
+
+
+bool make_path( const std::string & name )
+  {
+  const mode_t mode = S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
+  unsigned end = name.size();	// first slash before last component
+
+  while( end > 0 && name[end-1] == '/' ) --end;	// remove trailing slashes
+  while( end > 0 && name[end-1] != '/' ) --end;	// remove last component
+  while( end > 0 && name[end-1] == '/' ) --end;	// remove more slashes
+
+  unsigned index = 0;
+  while( index < end )
+    {
+    while( index < end && name[index] == '/' ) ++index;
+    unsigned first = index;
+    while( index < end && name[index] != '/' ) ++index;
+    if( first < index )
+      {
+      const std::string partial( name, 0, index );
+      struct stat st;
+      if( stat( partial.c_str(), &st ) == 0 )
+        { if( !S_ISDIR( st.st_mode ) ) return false; }
+      else if( mkdir( partial.c_str(), mode ) != 0 && errno != EEXIST )
+        return false;
+      }
+    }
+  return true;
+  }
