@@ -1,5 +1,5 @@
 /* Tarlz - Archiver with multimember lzip compression
-   Copyright (C) 2013-2020 Antonio Diaz Diaz.
+   Copyright (C) 2013-2021 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include <string>
 #include <vector>
 #include <fnmatch.h>
-#include <pthread.h>
+#include <pthread.h>	// for tarlz.h
 #include <stdint.h>
 
 #include "tarlz.h"
@@ -39,6 +39,8 @@ std::vector< std::string > patterns;		// list of patterns
 void Exclude::add_pattern( const std::string & arg )
   { patterns.push_back( arg ); }
 
+void Exclude::clear() { patterns.clear(); }
+
 
 bool Exclude::excluded( const char * const filename )
   {
